/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.actions;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.AbstractOperation;
import org.eclipse.core.commands.operations.IOperationHistory;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.resource.ImageDescriptor;

import eu.etaxonomy.taxeditor.UiUtil;

/**
 * @author p.ciardelli
 * @created 09.06.2008
 * @version 1.0
 */
public abstract class UndoableAction extends Action {
	private static final Logger logger = Logger.getLogger(UndoableAction.class);
	
	protected IUndoableOperation operation;
	private String text;
	
	protected UndoableAction(String text, ImageDescriptor image) {
		super(text);
		setImageDescriptor(image);
		this.text = text;
		
		operation = new UndoableOperation();
	}
	
	
	public void run() {	
		IOperationHistory operationHistory = UiUtil.getOperationHistory();
		IUndoContext undoContext = UiUtil.getWorkbenchUndoContext();
		operation.addContext(undoContext);
		try {
			operationHistory.execute(operation, null, null);
			operationHistory.add(operation);
		} catch (ExecutionException e) {
			e.printStackTrace();
		}
	}
	
	protected abstract IStatus executeOperation();
	protected abstract IStatus redoOperation();
	protected abstract IStatus undoOperation();
	
	class UndoableOperation extends AbstractOperation {

		public UndoableOperation() {
			super("'" + text + "'");
		}

		@Override
		public IStatus execute(IProgressMonitor monitor, IAdaptable info)
				throws ExecutionException {
			return executeOperation();
		}

		@Override
		public IStatus redo(IProgressMonitor monitor, IAdaptable info)
				throws ExecutionException {
			return redoOperation();
		}

		@Override
		public IStatus undo(IProgressMonitor monitor, IAdaptable info)
				throws ExecutionException {
			return undoOperation();
		}
	}
	
}
