/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.actions.cdm;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.resource.ImageDescriptor;

import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.SynonymRelationship;
import eu.etaxonomy.cdm.model.taxon.SynonymRelationshipType;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.model.CdmUtil;

/**
 * Add a new synonym to a taxon.
 * 
 * @author p.ciardelli
 * @created 06.05.2008
 * @version 1.0
 * 
 */
public class AddHomotypicSynonymToTaxonAction extends Action {
	private static final Logger logger = Logger
			.getLogger(AddHomotypicSynonymToTaxonAction.class);

	private static String text = "Create a homotypic synonym";
	private ImageDescriptor image = TaxEditorPlugin.getDefault()
			.getImageDescriptor(ITaxEditorConstants.HOMOTYPIC_SYN_ICON);

	Taxon taxon;
	Synonym synonym = null;
	TaxonNameBase synonymName = null;

	private AddHomotypicSynonymToTaxonAction() {
		super(text);
		setImageDescriptor(image);
	}

	public AddHomotypicSynonymToTaxonAction(TaxonNameBase synonymName,
			Taxon taxon) {
		this();
		this.synonymName = synonymName;
		this.taxon = taxon;
	}

	public AddHomotypicSynonymToTaxonAction(Synonym synonym, Taxon taxon) {
		this();
		this.synonym = synonym;
		this.taxon = taxon;
	}

	public AddHomotypicSynonymToTaxonAction(String unparsedName, Taxon taxon) {
		this(CdmUtil.parseFullReference(unparsedName, null, null), taxon);
	}

	public void run() {
		if (synonym != null) {
			taxon.addHomotypicSynonym(synonym, null, null);
		} else {
			if (synonymName != null) {
				SynonymRelationship synonymRelationship = taxon
						.addHomotypicSynonymName(synonymName, null, null);
				synonym = synonymRelationship.getSynonym();
			}
		}
	}

	public Synonym getSynonym() {
		return synonym;
	}
}