/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.actions.cdm;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.resource.ImageDescriptor;

import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;

/**
 * 1) Changes synonym name's homotypic group, 2) Deletes synonym from its accepted taxon,
 * 3) Creates a new synonym for the accepted taxon
 * @author p.ciardelli
 * @created 22.05.2008
 * @version 1.0
 */
public class ChangeSynonymHomotypicGroupAction extends Action {
	private static final Logger logger = Logger
			.getLogger(ChangeSynonymHomotypicGroupAction.class);

	private static String text = "Change synonym's homotypic group";
	private ImageDescriptor image = null;

	private Synonym synonym;
	private Taxon taxon;
	private HomotypicalGroup newHomotypicGroup;

	private ChangeSynonymHomotypicGroupAction() {
		super(text);
		setImageDescriptor(image);
	}
	
	public ChangeSynonymHomotypicGroupAction(Synonym synonym, Taxon taxon, HomotypicalGroup newHomotypicGroup) {
		this();
		
		this.synonym = synonym;
		this.taxon = taxon;
		this.newHomotypicGroup = newHomotypicGroup;
	}
	
	public void run() {
		
		if (synonym == null || synonym.getName() == null || 
				taxon == null || taxon.getName() == null) {
			return;
		}
		
		TaxonNameBase synonymName = synonym.getName();
		String synonymType = null;
				
		HomotypicalGroup oldHomotypicGroup = synonymName.getHomotypicalGroup();
		HomotypicalGroup taxonHomotypicGroup = taxon.getName().getHomotypicalGroup();
		
		new ChangeNameHomotypicGroup(synonymName, newHomotypicGroup).run();
				
//		// Has relation changed from homotypic to heterotypic?
//		if (oldHomotypicGroup.equals(taxonHomotypicGroup) && !newHomotypicGroup.equals(taxonHomotypicGroup)) {
//			new DeleteSynonymFromTaxonAction(synonym, taxon).run();
//			AddHeterotypicSynonymToTaxonAction addSynonymAction = new AddHeterotypicSynonymToTaxonAction(synonymName, taxon);
//			addSynonymAction.run();
//			synonym = addSynonymAction.getSynonym();
//		}
//
//		// Has relation changed from heterotypic to homotypic?
//		if (!oldHomotypicGroup.equals(taxonHomotypicGroup) && newHomotypicGroup.equals(taxonHomotypicGroup)) {
//			new DeleteSynonymFromTaxonAction(synonym, taxon).run();
//			AddHomotypicSynonymToTaxonAction addSynonymAction = new AddHomotypicSynonymToTaxonAction(synonymName, taxon);
//			addSynonymAction.run();
//			synonym = addSynonymAction.getSynonym();
//		}
		
		new DeleteSynonymFromAcceptedTaxaAction(synonym).run();
		if (newHomotypicGroup.equals(taxonHomotypicGroup)) {
			AddHomotypicSynonymToTaxonAction addSynonymAction = new AddHomotypicSynonymToTaxonAction(synonymName, taxon);
			addSynonymAction.run();
			synonym = addSynonymAction.getSynonym();			
		} else {
			AddHeterotypicSynonymToTaxonAction addSynonymAction = new AddHeterotypicSynonymToTaxonAction(synonymName, taxon);
			addSynonymAction.run();
			synonym = addSynonymAction.getSynonym();
		}
		
		firePropertyChange(ITaxEditorConstants.HOMOTYPIC_GROUP, null, synonym);		 
	}
}