/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.actions.cdm;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.resource.ImageDescriptor;

import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;

/**
 * @author p.ciardelli
 * @created 22.05.2008
 * @version 1.0
 */
public class ChangeSynonymToMisappliedNameAction extends Action {
	private static final Logger logger = Logger
			.getLogger(ChangeSynonymToMisappliedNameAction.class);

	private static String text = "Change synonym to misapplied name";
	private ImageDescriptor image = TaxEditorPlugin.getDefault()
			.getImageDescriptor(ITaxEditorConstants.MISAPPLIED_NAME_ICON);

	DeleteSynonymFromTaxonAction deleteSynonymAction;
	AddMisappliedNameToTaxonAction addMisappliedNameAction;

	private Synonym synonym;
	private Taxon taxon;
	private Taxon misappliedNameTaxon;
	
	private ChangeSynonymToMisappliedNameAction() {
		super(text);
		setImageDescriptor(image);
	}
	
	public ChangeSynonymToMisappliedNameAction(Synonym synonym, Taxon taxon) {
		this();
		this.synonym = synonym;
		this.taxon = taxon;
	}
	
	public void run() {
		TaxonNameBase name = synonym.getName();
		
		deleteSynonymAction = new DeleteSynonymFromTaxonAction(synonym, taxon);
		deleteSynonymAction.run();
		
		addMisappliedNameAction = new AddMisappliedNameToTaxonAction(name, taxon);
		addMisappliedNameAction.run();
		misappliedNameTaxon = addMisappliedNameAction.getMisappliedNameTaxon();
		
		firePropertyChange(ITaxEditorConstants.MISAPPLIED_NAME, null, misappliedNameTaxon);		 
	}

	public Taxon getMisappliedNameTaxon() {
		return misappliedNameTaxon;
	}
}