package eu.etaxonomy.taxeditor.actions.cdm;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.Action;

import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.UiUtil;
import eu.etaxonomy.taxeditor.navigation.TaxonomicTreeViewer;

/**
 * Commit any changes to this taxon to the CDM
 * 
 * @author p.ciardelli
 *
 */
public class SaveTaxonAction extends Action {
	private static final Logger logger = Logger.getLogger(SaveTaxonAction.class);
	
	Taxon taxon;
	
	public SaveTaxonAction(Taxon taxon) {
		this.taxon = taxon;
	}
	
	public void run() {
		// If this is a new taxon, add it to the recent names list;
		//  if it is already in the tree, remove then re-add it
		//  to force a re-sort
//		if (!TaxEditorPlugin.getDefault().getObservableRecentNamesList().contains(taxon)) {	
//			TaxEditorPlugin.getDefault().getObservableRecentNamesList().add(0, taxon);
//		}

		// If this action creates a root taxon, create the tax. tree
		TaxonomicTreeViewer treeViewer = UiUtil.getTreeViewer();
		if (treeViewer == null) {
			treeViewer = UiUtil.createTreeViewer();
		}		
		
		TaxEditorPlugin.getDefault().addSessionTaxon(taxon);
		
		// Save the taxon to the CDM
		TaxEditorPlugin.getDefault().getCdmApp().getTaxonService().saveTaxon(taxon);

		// Notify taxon listeners in case name has been updated - updates editor view tab and recent names
		taxon.firePropertyChange("name", null, null);

		
		// Open node on save
		treeViewer.revealTaxon(taxon);
		
	}
}