/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.actions.io;

import java.io.File;
import java.net.MalformedURLException;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.widgets.FileDialog;

import eu.etaxonomy.cdm.database.ICdmDataSource;
import eu.etaxonomy.cdm.io.common.ImportWrapper;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.UiUtil;

/**
 * @author p.ciardelli
 * @created 05.11.2008
 * @version 1.0
 */
public class ImportAction extends Action {
	private static final Logger logger = Logger.getLogger(ImportAction.class);

	private static String text = "Import ...";
	private ImageDescriptor image = null;
	public static final String ID = "eu.etaxonomy.taxeditor.actions.io.importaction"; //$NON-NLS-1$

	private FileDialog dialog;
	private File file;
	private ImportWrapper importWrapper;
	
	public ImportAction() {
		super(text);
		setImageDescriptor(image);
		setId(ID);
	}

	public ImportAction(ImportWrapper importWrapper) {
		this();
		
		this.importWrapper = importWrapper;
		setText(importWrapper.getLabel());
	}

	public void run() {

		// Use same title "Import FORMAT" for all message dialogs
		String title = "Import " + getText();
		
		// Make sure user is aware of the implications of an import
		boolean doProceed = MessageDialog.openConfirm(UiUtil.getShell(), title, 
				"Import may require a great deal of time and system resources.\n\n" +
				"Are you sure you would like to proceed?");
		if (!doProceed) {
			return;
		}
		
		// Get file from user
		dialog = new FileDialog(UiUtil.getShell());
		String filePath = dialog.open();
		file = new File(filePath);
				
		// Get current data source
		ICdmDataSource destination = TaxEditorPlugin.getDefault().getCdmDataSource();
		
		// Format file path 
		String source = null;
		try {
			source = file.toURI().toURL().toString();
		} catch (MalformedURLException e) {
			MessageDialog.openError(UiUtil.getShell(), title, "File could not be read.");
			return;
		}
		
		// Start import
		boolean isSuccessfulImport = importWrapper.invoke(source, destination, null);
		
		// Tell user whether import was a success or a dismal failure
		String msg = null;
		if (isSuccessfulImport) {
			MessageDialog.openInformation(UiUtil.getShell(), title, "Import successful");
		} else {
			MessageDialog.openError(UiUtil.getShell(), title, "Import was unsuccessful.");
		}
		
	}
}
