/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.actions.ui;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.AbstractOperation;
import org.eclipse.core.commands.operations.IOperationHistory;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;

import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.UiUtil;
import eu.etaxonomy.taxeditor.actions.cdm.DeleteMisappliedNameFromTaxonAction;
import eu.etaxonomy.taxeditor.editor.name.NameComposite;

/**
 * @author p.ciardelli
 * @created 04.06.2008
 * @version 1.0
 */
public class DeleteMisappliedNameCompositeAction extends Action {
	private static final Logger logger = Logger
			.getLogger(DeleteMisappliedNameCompositeAction.class);

	private static String text = "Delete misapplied name from taxon";
	private ImageDescriptor image = TaxEditorPlugin.getDefault()
			.getImageDescriptor(ITaxEditorConstants.ACTIVE_DELETE_ICON);

	private Taxon misappliedName;

	private Taxon taxon;
	private NameComposite composite;
	private IUndoableOperation operation;

	private DeleteMisappliedNameCompositeAction() {
		super(text);
		setImageDescriptor(image);
	}

	public DeleteMisappliedNameCompositeAction(NameComposite composite,	Taxon taxon) {
		this();

		if (composite.getData() instanceof Taxon) {
			this.misappliedName = (Taxon) composite.getData();
		} else {
			throw new IllegalArgumentException(
					"This action requires a composite with a Taxon in its data field.");
		}

		this.taxon = taxon;
		this.composite = composite;

		operation = new DeleteSynonymOperation();
	}

	public void run() {
		IOperationHistory operationHistory = UiUtil.getOperationHistory();
		IUndoContext undoContext = UiUtil.getWorkbenchUndoContext();
		operation.addContext(undoContext);
		try {
			operationHistory.execute(operation, null, null);
			operationHistory.add(operation);
		} catch (ExecutionException e) {
			e.printStackTrace();
		}
	}

	class DeleteSynonymOperation extends AbstractOperation {

		public DeleteSynonymOperation() {
			super("'" + text + "'");
		}

		@Override
		public IStatus execute(IProgressMonitor monitor, IAdaptable info)
				throws ExecutionException {
			// Delete synonym from taxon
			Action action = new DeleteMisappliedNameFromTaxonAction(misappliedName, taxon);
			action.addPropertyChangeListener(new IPropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent event) {
					if (event.getProperty().equals(
							ITaxEditorConstants.MISAPPLIED_NAME)) {
						// Delete namecomposite
						// TODO: put this in taxonFactory?
						composite.dispose();		
					}
				}
			});			
			action.run();
			
			return Status.OK_STATUS;
		}

		@Override
		public IStatus redo(IProgressMonitor monitor, IAdaptable info)
				throws ExecutionException {
			// TODO Auto-generated method stub
			return Status.OK_STATUS;
		}

		@Override
		public IStatus undo(IProgressMonitor monitor, IAdaptable info)
				throws ExecutionException {
			// TODO Auto-generated method stub
			return Status.OK_STATUS;
		}
	}
}
