/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.editor;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.Assert;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.forms.IManagedForm;

import eu.etaxonomy.cdm.model.taxon.Taxon;

/** 
 * Formats <code>GroupComposite</code> with cosmetic and layout properties specific to the
 * Editor. This should be used to maintain a consistent look and feel for all Editor 
 * freetext area group components, such as <code>HomotypicalGroup</code>.
 * <p>
 * Its <code>IManagedForm</code> is required to have a <code>Taxon</code> in its 
 * own <code>getData()</code>.
 * </p> 
 * 
 * @author p.ciardelli
 * @created 02.06.2008
 * @version 1.0
 */
public class EditorGroupComposite extends GroupComposite {
	private static final Logger logger = Logger.getLogger(EditorGroupComposite.class);
	
	private IManagedForm managedForm;
	private Taxon taxon;
	
	public EditorGroupComposite(Composite parent, IManagedForm managedForm, Object data) {
		this(parent, managedForm);
		
		setData(data);
	}

	public EditorGroupComposite(Composite parent, IManagedForm managedForm) {
		super(parent);

		this.managedForm = managedForm;
		
		Object formData = managedForm.getForm().getBody().getData();
		Assert.isTrue(formData instanceof Taxon, 
				"Managed form must have a Taxon in its data field.");
		taxon = (Taxon) formData;
		
		createBorderSupport();
	}

	private void createBorderSupport() {
		new CompositeBorderDecorator(this, managedForm);
	}
	
	public IManagedForm getManagedForm() {
		return managedForm;
	}
	
	public Taxon getTaxon() {
		return taxon;
	}
}