/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.navigation;

import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.ManagedForm;
import org.eclipse.ui.forms.events.HyperlinkAdapter;
import org.eclipse.ui.forms.events.HyperlinkEvent;
import org.eclipse.ui.forms.widgets.Hyperlink;
import org.eclipse.ui.forms.widgets.TableWrapLayout;
import org.eclipse.ui.part.ViewPart;

import com.swtdesigner.ResourceManager;
import com.swtdesigner.SWTResourceManager;

import eu.etaxonomy.cdm.database.ICdmDataSource;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.UiUtil;
import eu.etaxonomy.taxeditor.actions.ui.OpenTaxonEditorAction;
import eu.etaxonomy.taxeditor.datasource.CdmDataSourceDialog;

/**
 * The left navigation pane.
 * 
 * @author p.ciardelli
 * @created 27.05.2008
 * @version 1.0
 */
public class TaxonomicTreeView extends ViewPart {
	private static final Logger logger = Logger.getLogger(TaxonomicTreeView.class);
	
	private Composite dbComposite;
	private Composite taxTreeComposite = null;
	private Composite top = null;
	public static final String ID = "eu.etaxonomy.taxeditor.navigation.taxonomictreeview"; //$NON-NLS-1$
	
	private TaxonomicTreeViewer treeViewer;

	private IManagedForm linkForm;

	private Menu dataSourceMenu;
	
	/**
	 * Create contents of the view part
	 * @param parent
	 */
	@Override
	public void createPartControl(Composite parent) {
				
		parent.setLayout(new FillLayout());
		GridLayout gridLayout = new GridLayout();
		gridLayout.horizontalSpacing = 0;
		gridLayout.marginWidth = 0;
		gridLayout.marginHeight = 0;
		gridLayout.verticalSpacing = 0;	
				
		top = new Composite(parent, SWT.NONE);
		top.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
		top.setLayout(gridLayout);

		dbComposite = new Composite(top, SWT.NONE);
		dbComposite.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
		final GridLayout gridLayout_1 = new GridLayout();
		gridLayout_1.numColumns = 2;
		dbComposite.setLayout(gridLayout_1);
		final Label lblDataSourceTitle = new Label(dbComposite, SWT.NONE);
		lblDataSourceTitle.setFont(SWTResourceManager.getFont("Arial", 14, SWT.BOLD));
		lblDataSourceTitle.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
		lblDataSourceTitle.setAlignment(SWT.CENTER);
		lblDataSourceTitle.setText(" Local");		
		
		final Label lblDatasourceMenu = new Label(dbComposite, SWT.NONE);
		lblDatasourceMenu.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false));
		lblDatasourceMenu.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
		lblDatasourceMenu.setImage(ResourceManager.getPluginImage(TaxEditorPlugin.getDefault(), "icons/db.gif"));
		lblDatasourceMenu.setImage(TaxEditorPlugin.getDefault().getImageRegistry().get(ITaxEditorConstants.DB_ICON));

		dataSourceMenu = new Menu(lblDatasourceMenu);
		lblDatasourceMenu.setMenu(dataSourceMenu);

//		createDataSourceMenu();

		final Label lblDataSource = new Label(dbComposite, SWT.NONE);
		final GridData gd_lblDataSource = new GridData(SWT.LEFT, SWT.CENTER, false, false, 2, 1);
		gd_lblDataSource.heightHint = 16;
		lblDataSource.setLayoutData(gd_lblDataSource);
		lblDataSource.setFont(SWTResourceManager.getFont("Arial", 8, SWT.NONE));
		lblDataSource.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
		lblDataSource.setAlignment(SWT.CENTER);
		lblDataSource.setText("  Datasource: local h2 database");

		taxTreeComposite = new Composite(top, SWT.NONE);
		taxTreeComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		taxTreeComposite.setLayout(new FillLayout());
		taxTreeComposite.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
		
		// If there are no root taxa, provide a link to create this tree's initial root taxon
		if (TaxEditorPlugin.getDefault().getSessionRootTaxa().size() == 0) {
			
			linkForm = new ManagedForm(taxTreeComposite);
			Composite linkBody = linkForm.getForm().getBody();
			
			TableWrapLayout linkLayout = new TableWrapLayout();
			linkLayout.leftMargin = 20;
			linkLayout.rightMargin = 20;
			linkBody.setLayout(linkLayout);
			linkBody.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
			
			Hyperlink link = linkForm.getToolkit().createHyperlink(linkBody, 
					"Your data source is empty. Click here to create a root taxon.", SWT.WRAP);
			link.addHyperlinkListener(new HyperlinkAdapter() {

				@Override
				public void linkActivated(HyperlinkEvent e) {
					new OpenTaxonEditorAction().run();
				}				
			});
			
			treeViewer = null;
			
		} else {
			createTreeViewer();
		}

		createActions();
		initializeToolBar();
		initializeMenu();
		
//		TaxEditorPlugin.getDefault().setTaxonomicTree(this);
	}
	
	private void createDataSourceMenu() {
		
		// Get current datasource and list of all data sources 
		ICdmDataSource sessionDataSource = TaxEditorPlugin.getDefault()
				.getCdmDataSource();
		List<ICdmDataSource> dataSources = 
			TaxEditorPlugin.getDefault().getDataSources();
		
		// Create menu items, putting a check next to the current one
		for (ICdmDataSource dataSource : dataSources) {
			MenuItem item = new MenuItem(dataSourceMenu, SWT.CHECK);
			
			if (dataSource.equals(sessionDataSource)) {
				item.setSelection(true);
			}
			
			String text = dataSource.getName();
			if (dataSource.getDatabase() != null) {
				text += " (" + dataSource.getDatabase() + ")";
			}
			item.setText(text);
		}

		new MenuItem(dataSourceMenu, SWT.SEPARATOR);

		final MenuItem newDataSourceMenuItem = new MenuItem(dataSourceMenu, SWT.NONE);
		newDataSourceMenuItem.setText("Edit data sources ...");
		newDataSourceMenuItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
			  	Shell shell = UiUtil.getShell();
				Dialog dialog = new CdmDataSourceDialog(shell);
				dialog.open();
			}
		});		
	}

	public TaxonomicTreeViewer createTreeViewer() {
		
		// Dispose link to new taxon as necessary
		if (linkForm != null) {
			linkForm.getForm().dispose();
		}
		
		// Create tree
		
		treeViewer = new TaxonomicTreeViewer(taxTreeComposite);
		
		// TODO move these into this class
		((TaxonomicTreeViewer) treeViewer).createDoubleClickListener();
		((TaxonomicTreeViewer) treeViewer).createMenu();
		((TaxonomicTreeViewer) treeViewer).createQuickAdd();
		((TaxonomicTreeViewer) treeViewer).createDragAndDrop();
				
		taxTreeComposite.layout();
		
		return treeViewer;
	}
	
	private void createActions() {
		// Create the actions
	}

	/**
	 * Initialize the toolbar
	 */
	private void initializeToolBar() {
		IToolBarManager toolbarManager = getViewSite().getActionBars()
				.getToolBarManager();
	}

	/**
	 * Initialize the menu
	 */
	private void initializeMenu() {
		IMenuManager menuManager = getViewSite().getActionBars()
				.getMenuManager();
	}

	@Override
	public void setFocus() {
		// Set the focus
	}
	
	/**
	 * Expose TreeViewer, i.e. to manipulate tree nodes
	 * @return
	 */
	public TaxonomicTreeViewer getTreeViewer() {
		return treeViewer;
	}		
}
