/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.propertysheet.reference;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;

import org.apache.log4j.Logger;

import eu.etaxonomy.cdm.model.reference.Article;
import eu.etaxonomy.cdm.model.reference.BibtexReference;
import eu.etaxonomy.cdm.model.reference.Book;
import eu.etaxonomy.cdm.model.reference.BookSection;
import eu.etaxonomy.cdm.model.reference.Generic;
import eu.etaxonomy.cdm.model.reference.PrintedUnitBase;
import eu.etaxonomy.cdm.model.reference.PublicationBase;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.cdm.model.reference.SectionBase;
import eu.etaxonomy.cdm.model.reference.StrictReferenceBase;

/**
 * Quick and dirty subclass of <code>ReferencePropertySource</code>.
 * <p>
 * TODO: reprogram to only use <code>ReferenceBase</code> classes which implement
 * <code>INomenclaturalReference</code>.
 *  
 * @author p.ciardelli
 * @created 12.11.2008
 * @version 1.0
 */
public class NomenclaturalReferencePropertySource extends
		ReferencePropertySource {
	private static final Logger logger = Logger
			.getLogger(NomenclaturalReferencePropertySource.class);
	
	public NomenclaturalReferencePropertySource(ReferenceBase reference) {
		super(reference);
	}
	
	protected void populateReferenceTypes() {
		
		// LinkedHashMap maintains insertion order
		referenceTypeMap = new LinkedHashMap<Class, String>();
		
		referenceTypeMap.put(ReferenceBase.class, "");
		referenceTypeMap.put(BibtexReference.class, "BibTeX Reference");
		referenceTypeMap.put(Article.class, "Article");
		referenceTypeMap.put(Generic.class, "Generic");
		referenceTypeMap.put(Book.class, "Book");
		referenceTypeMap.put(BookSection.class, "Book Section");
	}
	
	protected void initDescriptors() {
		
		List<String> displayFields = new ArrayList<String>();
	
		// Drop-down menu to change reference type
		displayFields.add(P_ID_REFERENCETYPE);
		
		// ReferenceBase fields
		displayFields.add(P_ID_AUTHORTEAM);
		displayFields.add(P_ID_YEAR);
		displayFields.add(P_ID_CITATION);
		displayFields.add(P_ID_URI);
		
		Class referenceClass = reference.getClass();
		
		if (reference instanceof BibtexReference) {
						
			displayFields.add(P_ID_BIBTEX_ENTRYTYPE);
			displayFields.add(P_ID_JOURNAL);
			displayFields.add(P_ID_BOOKTITLE);
			displayFields.add(P_ID_CHAPTER);
			displayFields.add(P_ID_TITLE);
			displayFields.add(P_ID_SERIES);
			displayFields.add(P_ID_EDITION);
			displayFields.add(P_ID_VOLUME);
			displayFields.add(P_ID_NUMBER);
			displayFields.add(P_ID_PAGES);
			displayFields.add(P_ID_ANNOTE);
			displayFields.add(P_ID_EDITOR);
			displayFields.add(P_ID_INSTITUTION);
			displayFields.add(P_ID_SCHOOL);
			displayFields.add(P_ID_ORGANIZATION);
			displayFields.add(P_ID_PUBLISHER);
			displayFields.add(P_ID_ADDRESS);
			displayFields.add(P_ID_HOWPUBLISHED);
			displayFields.add(P_ID_REPORTTYPE);
			displayFields.add(P_ID_MONTH);
			displayFields.add(P_ID_EPRINT);
			displayFields.add(P_ID_NOTE);
			displayFields.add(P_ID_CROSSREF);
		}
	
		if (reference instanceof StrictReferenceBase) {
			
			displayFields.add(P_ID_DATEPUBLISHED);
			displayFields.add(P_ID_TITLE);
	
			if (referenceClass == Article.class) {
				displayFields.add(P_ID_INJOURNAL);
				displayFields.add(P_ID_PAGES);
				displayFields.add(P_ID_SERIES);
				displayFields.add(P_ID_VOLUME);
			}
	
			if (referenceClass == Generic.class) {
				displayFields.add(P_ID_PAGES);
				displayFields.add(P_ID_SERIES);
				displayFields.add(P_ID_VOLUME);
				displayFields.add(P_ID_EDITOR);
				displayFields.add(P_ID_PLACEPUBLISHED);
				displayFields.add(P_ID_PUBLISHER);
			}
			
			if (reference instanceof PublicationBase) {
	
				displayFields.add(P_ID_PLACEPUBLISHED);
				displayFields.add(P_ID_PUBLISHER);
				
				if (reference instanceof PrintedUnitBase) {
					
					displayFields.add(P_ID_EDITOR);
					displayFields.add(P_ID_INSERIES);
					displayFields.add(P_ID_PAGES);
					displayFields.add(P_ID_SERIESPART);
					displayFields.add(P_ID_VOLUME);
					
					if (referenceClass == Book.class) {
						displayFields.add(P_ID_ISBN);
						displayFields.add(P_ID_EDITION);
					}
				}
			}
			
			if (reference instanceof SectionBase) {
				
				displayFields.add(P_ID_PAGES);	
				displayFields.add(P_ID_PRINTEDUNIT);
				
				if (referenceClass == BookSection.class) {
					displayFields.add(P_ID_INBOOK);
				}
			}
		}
		
		for (String field : displayFields) {
			addDescriptor(field);
		}
	}
}
